HedgewarsScriptLoad("/Scripts/SimpleMission.lua")
HedgewarsScriptLoad("/Scripts/Locale.lua")
SimpleMission({
	missionTitle = loc("Tentacle Terror"),
	initVars = {
		TurnTime = 25000,
		MinesTime = 0,
		Map = "SB_Tentacles",
		Theme = "Hell",
		GameFlags = gfDisableWind
	},
	ammoConfig = {
		[amParachute] = { count = 1 },
		[amRope] = { count = 1, weaponsInCrate = 1 },
		[amBazooka] = { weaponsInCrate = 2 },
	},
	wind = -50,
	teams = {
		{ isMissionTeam = true,
		hogs = { {
			faceLeft = true,
			health = 30,
			x = 1210, y = 1446,
			} }
		},
		{ name = loc("Heavenly Defense"),
		flag = "cm_fcw",
		grave = "Ghost",
		hogs = { {
			name = loc("Fallen Angel"),
			health = 100,
			x = 1723, y = 1484,
			hat = "angel",
			botLevel = 1,
			ammo = { [amAirAttack] = 100 } } }
		}
	},
	gears = {
		{ type = gtMine, x=2205, y=1230 },
		{ type = gtMine, x=2339, y=1274 },
		{ type = gtMine, x=2372, y=1239 },
		{ type = gtMine, x=2405, y=1228 },
		{ type = gtMine, x=2437, y=1204 },
		{ type = gtMine, x=2463, y=1190 },
		{ type = gtMine, x=2488, y=1180 },
		{ type = gtMine, x=2517, y=1169 },
		{ type = gtMine, x=2546, y=1161 },
		{ type = gtMine, x=2574, y=1154 },
		{ type = gtMine, x=2604, y=1148 },
		{ type = gtMine, x=2636, y=1141 },
		{ type = gtMine, x=2176, y=1211 },
		{ type = gtMine, x=2152, y=1198 },
		{ type = gtMine, x=2120, y=1184 },
		{ type = gtMine, x=2090, y=1174 },
		{ type = gtMine, x=2058, y=1166 },
		{ type = gtMine, x=2036, y=1162 },
		{ type = gtMine, x=1995, y=1158 },
		{ type = gtMine, x=1957, y=1157 },
		{ type = gtMine, x=1891, y=1162 },
		{ type = gtMine, x=1838, y=1173 },
		{ type = gtMine, x=1799, y=1181 },
		{ type = gtMine, x=1763, y=1192 },
		{ type = gtMine, x=1722, y=1210 },
		{ type = gtMine, x=1676, y=1232 },
		{ type = gtMine, x=1653, y=1247 },
		{ type = gtMine, x=1626, y=1269 },
		{ type = gtMine, x=1595, y=1303 },
		{ type = gtMine, x=1563, y=1287 },
		{ type = gtMine, x=1536, y=1258 },
		{ type = gtMine, x=1522, y=1231 },
		{ type = gtMine, x=1508, y=1184 },
		{ type = gtMine, x=1496, y=1139 },
		{ type = gtMine, x=2729, y=1285 },
		{ type = gtMine, x=2799, y=1329 },
		{ type = gtMine, x=2851, y=1383 },
		{ type = gtMine, x=2912, y=1369 },
		{ type = gtMine, x=2668, y=1261 },
		{ type = gtMine, x=2903, y=1219 },
		{ type = gtMine, x=2916, y=1171 },
		{ type = gtMine, x=3143, y=1186 },
		{ type = gtMine, x=3074, y=1202 },
		{ type = gtMine, x=1046, y=1644 },
		{ type = gtMine, x=989, y=1719 },
		{ type = gtMine, x=850, y=1827 },
		{ type = gtMine, x=1000, y=1203 },
		{ type = gtMine, x=3047, y=1040 },
		{ type = gtMine, x=3200, y=1592 },
		{ type = gtMine, x=3257, y=1638 },
		{ type = gtMine, x=3314, y=1698 },
		{ type = gtMine, x=3364, y=1769 },
		{ type = gtMine, x=3423, y=1905 },
		{ type = gtMine, x=1750, y=1584 },
		{ type = gtMine, x=1768, y=1590 },
		{ type = gtMine, x=1801, y=1725 },
		{ type = gtMine, x=1832, y=1698 },
		{ type = gtMine, x=1854, y=1660 },
		{ type = gtMine, x=1888, y=1701 },
		{ type = gtMine, x=1917, y=1675 },
		{ type = gtMine, x=1957, y=1637 },
		{ type = gtMine, x=1985, y=1573 },
		{ type = gtMine, x=2004, y=1545 },
		{ type = gtMine, x=2051, y=1524 },
		{ type = gtMine, x=2075, y=1487 },
		{ type = gtMine, x=1796, y=1670 },
		{ type = gtCase, crateType = "supply", x = 3484, y = 1950, ammoType = amBazooka },
		{ type = gtCase, crateType = "supply", x = 1139, y = 1062, ammoType = amBazooka },
		{ type = gtCase, crateType = "supply", x = 2613, y = 1545, ammoType = amDrillStrike },
		{ type = gtCase, crateType = "supply", x = 3029, y = 1601, ammoType = amBlowTorch },
		{ type = gtCase, crateType = "supply", x = 707, y = 1957, ammoType = amRope },
	},
})
